'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extend2 = require('lodash/extend');

var _extend3 = _interopRequireDefault(_extend2);

var _merge2 = require('lodash/merge');

var _merge3 = _interopRequireDefault(_merge2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _createReactClass = require('create-react-class');

var _createReactClass2 = _interopRequireDefault(_createReactClass);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _loreReactFormsBootstrap = require('lore-react-forms-bootstrap');

var _formMap = require('./formMap');

var _formMap2 = _interopRequireDefault(_formMap);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = (0, _createReactClass2.default)({
  displayName: 'Wizard',

  propTypes: {
    steps: _propTypes2.default.array.isRequired
  },

  getInitialState: function getInitialState() {
    var _props = this.props,
        model = _props.model,
        data = _props.data;


    return {
      key: 0,
      stepIndex: 0,
      data: (0, _merge3.default)({}, data, model.data),
      isSaving: false,
      request: null,
      hasError: false
    };
  },


  request: function request(data) {
    var modelName = this.props.modelName;
    var model = this.props.model;


    return lore.actions[modelName].destroy(model).payload;
  },

  onSubmit: function onSubmit(newData) {
    var data = this.state.data;

    var nextData = (0, _extend3.default)({}, data, newData);

    this.setState({
      isSaving: true,
      hasError: false,
      data: nextData,
      request: this.request(nextData)
    });
  },

  onRequestSuccess: function onRequestSuccess(request) {
    var steps = this.props.steps;

    var lastStepIndex = steps.length - 1;

    this.setState({
      isSaving: false,
      request: request,
      hasError: false,
      stepIndex: lastStepIndex
    });
  },

  onRequestError: function onRequestError(request) {
    this.setState({
      isSaving: false,
      request: request,
      hasError: true
    });
  },

  onResetWizard: function onResetWizard() {
    this.setState((0, _extend3.default)(this.getInitialState(), {
      key: this.state.key + 1
    }));
  },

  onNext: function onNext(newData, newStepIndex) {
    var _state = this.state,
        data = _state.data,
        stepIndex = _state.stepIndex;


    this.setState({
      stepIndex: newStepIndex || stepIndex + 1,
      data: (0, _extend3.default)({}, data, newData)
    });
  },

  onPrevious: function onPrevious(newData, newStepIndex) {
    var _state2 = this.state,
        data = _state2.data,
        stepIndex = _state2.stepIndex;


    this.setState({
      stepIndex: newStepIndex || stepIndex - 1,
      data: (0, _extend3.default)({}, data, newData)
    });
  },

  onChangeStep: function onChangeStep(newData, newStepIndex) {
    var data = this.state.data;


    this.setState({
      stepIndex: newStepIndex,
      data: (0, _extend3.default)({}, data, newData)
    });
  },


  render: function render() {
    var _this = this;

    var modelName = this.props.modelName;
    var _props2 = this.props,
        steps = _props2.steps,
        schema = _props2.schema,
        fieldMap = _props2.fieldMap,
        actionMap = _props2.actionMap;
    var _state3 = this.state,
        key = _state3.key,
        data = _state3.data,
        stepIndex = _state3.stepIndex,
        request = _state3.request,
        isSaving = _state3.isSaving,
        hasError = _state3.hasError;


    var stepConfig = steps[stepIndex];
    var form = _formMap2.default[stepConfig.form];

    var step = form.render({
      modelName: modelName,
      data: data,
      schema: schema,
      fieldMap: fieldMap,
      actionMap: actionMap,
      hasError: hasError,
      request: request,
      callbacks: {
        onNext: function onNext(newData, nextStep) {
          _this.onNext(newData, nextStep);
        },
        onPrevious: function onPrevious(newData, nextStep) {
          _this.onPrevious(newData, nextStep);
        },
        onSubmit: function onSubmit(newData) {
          _this.onSubmit(newData);
        },
        onResetWizard: this.onResetWizard
      }
    }, stepConfig);

    return _react2.default.createElement(
      _loreReactFormsBootstrap.Overlay,
      { isVisible: isSaving },
      _react2.default.createElement(
        'div',
        { key: key },
        request && isSaving ? _react2.default.createElement(_loreReactFormsBootstrap.Request, {
          request: request,
          reducer: modelName,
          onSuccess: this.onRequestSuccess,
          onError: this.onRequestError
        }) : null,
        step
      )
    );
  }

});