'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DestroyWizardBlueprint = exports.DestroyOverlayBlueprint = exports.DestroyOptimisticBlueprint = exports.UpdateWizardBlueprint = exports.UpdateOverlayBlueprint = exports.UpdateOptimisticBlueprint = exports.CreateWizardBlueprint = exports.CreateOverlayBlueprint = exports.CreateOptimisticBlueprint = exports.config = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _Optimistic = require('./blueprints/create/Optimistic');

Object.defineProperty(exports, 'CreateOptimisticBlueprint', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_Optimistic).default;
  }
});

var _Overlay = require('./blueprints/create/Overlay');

Object.defineProperty(exports, 'CreateOverlayBlueprint', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_Overlay).default;
  }
});

var _Wizard = require('./blueprints/create/Wizard');

Object.defineProperty(exports, 'CreateWizardBlueprint', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_Wizard).default;
  }
});

var _Optimistic2 = require('./blueprints/update/Optimistic');

Object.defineProperty(exports, 'UpdateOptimisticBlueprint', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_Optimistic2).default;
  }
});

var _Overlay2 = require('./blueprints/update/Overlay');

Object.defineProperty(exports, 'UpdateOverlayBlueprint', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_Overlay2).default;
  }
});

var _Wizard2 = require('./blueprints/update/Wizard');

Object.defineProperty(exports, 'UpdateWizardBlueprint', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_Wizard2).default;
  }
});

var _Optimistic3 = require('./blueprints/destroy/Optimistic');

Object.defineProperty(exports, 'DestroyOptimisticBlueprint', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_Optimistic3).default;
  }
});

var _Overlay3 = require('./blueprints/destroy/Overlay');

Object.defineProperty(exports, 'DestroyOverlayBlueprint', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_Overlay3).default;
  }
});

var _Wizard3 = require('./blueprints/destroy/Wizard');

Object.defineProperty(exports, 'DestroyWizardBlueprint', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_Wizard3).default;
  }
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _forms = require('./loaders/forms');

var _forms2 = _interopRequireDefault(_forms);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; } /* eslint no-param-reassign: "off" */

exports.config = _config2.default;
exports.default = {

  dependencies: ['models'],

  defaults: {
    forms: _config2.default
  },

  load: function load(lore) {
    var models = lore.loader.loadModels();
    var formSchemas = _forms2.default.load();
    var _lore$config$forms = lore.config.forms,
        defaultBlueprint = _lore$config$forms.defaultBlueprint,
        defaultSchema = _lore$config$forms.defaultSchema,
        blueprints = _lore$config$forms.blueprints,
        schemas = _lore$config$forms.schemas,
        fieldMap = _lore$config$forms.fieldMap,
        actionMap = _lore$config$forms.actionMap;


    lore.forms = {};

    /*
     * Step 1:
     * Create convention-based forms from models
     */
    _lodash2.default.mapValues(models, function (modelConfig, modelName) {
      lore.forms[modelName] = lore.forms[modelName] || {};

      var formsConfig = modelConfig.forms || {};

      function generateForm(blueprints, method) {
        return function form() {
          var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
          var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

          var _props$blueprint = props.blueprint,
              blueprintName = _props$blueprint === undefined ? defaultBlueprint : _props$blueprint,
              _props$schema = props.schema,
              schemaName = _props$schema === undefined ? defaultSchema : _props$schema,
              other = _objectWithoutProperties(props, ['blueprint', 'schema']);

          var config = formsConfig[method] || {};
          var blueprint = blueprints[blueprintName];
          var blueprintProps = _extends({
            modelName: modelName,
            schema: schemas[schemaName],
            fieldMap: fieldMap,
            actionMap: actionMap
          }, config, other);

          return _react2.default.createElement(blueprint, blueprintProps);
        };
      }

      _lodash2.default.assign(lore.forms[modelName], {
        create: function create() {
          var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
          var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

          return generateForm(blueprints.create, 'create')(props, options);
        },
        destroy: function destroy(model) {
          var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
          var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

          props.model = model;
          return generateForm(blueprints.destroy, 'destroy')(props, options);
        },
        update: function update(model) {
          var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
          var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

          props.model = model;
          return generateForm(blueprints.update, 'update')(props, options);
        }
      });
    });

    /*
     * Step 2:
     * Override conventions with any forms defined in /forms, ignoring any
     * folders that start with "_"
     */
    _lodash2.default.mapKeys(formSchemas, function (folderSchema, folderName) {
      // ignore any folders that begin with underscore
      if (_lodash2.default.startsWith(folderName, '_')) {
        return;
      }

      // if the result is not an object, this hook has no idea what to do
      // with it, so bail. This hook assumes a double folder structure, such
      // as src/forms/tweet/post.js or src/forms/tweet/post/index.js, and
      // has no idea what to do if the first result is not a folder named after
      // a model
      if (!_lodash2.default.isPlainObject(folderSchema)) {
        return;
      }

      lore.forms[folderName] = lore.forms[folderName] || {};

      _lodash2.default.mapKeys(folderSchema, function (schema, fileName) {
        var Component = null;

        if (_lodash2.default.startsWith(fileName, '_')) {
          return;
        }

        // if the result is a component, use that as the form
        if (_lodash2.default.isFunction(schema)) {
          Component = schema;
        }

        // if the result is an object, then it's a folder, so use the index file
        // as the component if one exists
        if (_lodash2.default.isPlainObject(schema) && _lodash2.default.isFunction(schema.index)) {
          Component = schema.index;
        }

        // if neither condition is true, this hook has no idea what to do, so bail
        if (!Component) {
          return;
        }

        // set the form to the component we found
        lore.forms[folderName][fileName] = Component;

        // if the name of the file is create, wrap it in a function so it mirrors
        // the behavior of the blueprint, so it can be invoked as a function
        if (fileName === 'create') {
          lore.forms[folderName][fileName] = function () {
            var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
            var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

            return _react2.default.createElement(Component, props);
          };
        }

        // if the name of the file is update or destroy, wrap it in a function so it
        // mirrors the behavior of the blueprint, so it can be invoked as a function
        // with the model we should update or destroy
        if (fileName === 'destroy' || fileName === 'update') {
          lore.forms[folderName][fileName] = function (model) {
            var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
            var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

            props.model = model;
            return _react2.default.createElement(Component, props);
          };
        }
      });
    });
  }

};